@@ -1,8 +1,8 @@
-test_that("validate_config() checks field value", {
+test_that("validate_config() checks option fields", {
   def <- list(
     list(
       name = "field",
-      options = c("apples", "bananas"),
+      validator = option_validator("apples", "bananas"),
       required = FALSE
     )
   )
@@ -12,11 +12,25 @@ test_that("validate_config() checks field value", {
   expect_error(validate_config(def, list(field = "cherries")))
 })
 
+test_that("validate_config() checks integer fields", {
+  def <- list(
+    list(
+      name = "field",
+      validator = positive_integer_validator,
+      required = FALSE
+    )
+  )
+  expect_silent(validate_config(def, list()))
+  expect_silent(validate_config(def, list(field = 1L)))
+  expect_error(validate_config(def, list(field = 1.)))
+  expect_error(validate_config(def, list(field = "1")))
+})
+
 test_that("validate_config() checks for required fields", {
   def <- list(
     list(
       name = "field",
-      options = c("apples", "bananas"),
+      options = positive_integer_validator,
       required = TRUE
     )
   )
